{-----------------------------------------------------------------------------
 Unit Name: COCOTreeBuilder
 Author:     pat connors
 Purpose: Contructs the GUI Tree from a CompilerObj
-----------------------------------------------------------------------------}


unit COCOTreeBuilder;

interface

uses
  Windows, Messages, SysUtils, Classes, ComCtrls,
  CompilerObj, CocoProductions, CocoProduction, CocoProductionPath,
  CocoToken, pbStrLib;

type
  TCOCOTreeBuilder = class(TObject)
  private
    FTreeView: TTreeView;
    FCompiler: TCompiler;
    FCompilerNode,
    FDelphiNode,
    fPrivateNode,
    fGlobalNode,
    fProtectedNode,
    fUsesNode,
    fPublicNode,
    fPublishedNode,
    fCreateNode,
    fDestroyNode,
    fTokensNode,
    fPragmasNode,
    fIgnoreNode,
    fCharactersNode,
    fErrorsNode,
    fNamesNode,
    fProductionsNode: TTreeNode;
    FArbitraryCodeNode: TTreeNode;
    procedure SetTreeView(const Value: TTreeView);
    procedure SetCompiler(const Value: TCompiler);
    procedure AddPrimaryNode;
    procedure AddCompilerNode;
    procedure AddDelphiNode;
    procedure AddCharactersNode;
    procedure AddTokensNode;
    procedure AddNamesNode;
    procedure AddIgnoreNode;
    procedure AddErrorsNode;
    procedure AddPragmasNode;
    procedure AddFramesNode;
    procedure AddProductionsNode;
    procedure AddCommentsNode;
    //    procedure AddHeaderNode;
    //    procedure AddOriginalATGNode;
    procedure KillCompilerNode;
    //    procedure AddChangeHistoryNode;
    function AddCocoNode(const NodeText: string; ParentNode: TTreeNode;
      Data: TObject): TTreeNode;
    //    procedure AddDelphiMembers(Node: TTreeNode; MEmberText: string);
    procedure SetArbitraryCodeNode(const Value: TTreeNode);
    function GetProdCount: Integer;
  protected
    { Protected declarations }
  public
    { Public declarations }
    function Execute: boolean;
    function GetProdNodeNamesList: string;
  published
    { Published declarations }
    property TreeView: TTreeView read FTreeView write SetTreeView;
    property Compiler: TCompiler read FCompiler write SetCompiler;
    property ArbitraryCodeNode: TTreeNode read FArbitraryCodeNode write SetArbitraryCodeNode;
    property ProdCount: Integer read GetProdCount;
  end;

implementation

uses CocoTokens;

{ TCOCOTreeBuilder }

{-----------------------------------------------------------------------------
  Procedure: TCOCOTreeBuilder.Execute
  Arguments: const ATGText: String
  Result:    Boolean
  Description
  Walks the fCompiler objects, filling out the treeview nodes as it goes.

-----------------------------------------------------------------------------}

function TCOCOTreeBuilder.Execute: boolean;
begin
  TreeView.Items.BeginUpdate;
  try
    //    TreeView.Items.Clear;
    AddCompilerNode;
    AddDelphiNode;
    AddCharactersNode;
    AddIgnoreNode;
    AddTokensNode;
    AddNamesNode;
    AddPragmasNode;
    AddProductionsNode;
    TreeView.Items[0].Expand(False);
    FCompilerNode.Expand(False);
    Result := True;
  finally
    TreeView.Items.EndUpdate;
  end;
end;

procedure TCOCOTreeBuilder.AddCharactersNode;
  //var
  //  idx: integer;
  //  S: string;
begin
  fCharactersNode := AddCocoNode('Characters', FCompilerNode, FCompiler);
  fCharactersNode.ImageIndex := 19;
  fCharactersNode.SelectedIndex := 19;
  fCharactersNode.StateIndex := 19;
  //  with FCompiler.CharactersSection.CharacterText do
  //    for idx := 0 to Count - 1 do
  //    begin
  //      S := Names[idx];
  //      AddCocoNode(S, fCharactersNode, nil);
  //    end;
end;

function TCOCOTreeBuilder.AddCocoNode(const NodeText: string;
  ParentNode: TTreeNode; Data: TObject): TTreeNode;
var
  P: Pointer absolute Data;
  S: string;
begin
  S := Trim(NodeText);
  { TODO : Need to determine if the node already exists - if so, then use it instead of creating a new one. }
  Result := TreeView.Items.AddChildObject(ParentNode, S, P);
  Result.ImageIndex := 25;
  Result.StateIndex := 25;
  Result.SelectedIndex := 25;
end;

procedure TCOCOTreeBuilder.AddCompilerNode;
begin
  AddPrimaryNode;
  Assert(TreeView.Items.Count > 0);

  //Determine if this next node already exists
  // - if so, then delete it from the tree
  KillCompilerNode;

  FCompilerNode := AddCocoNode(FCompiler.CompilerName + ' : ' + FCompiler.FileName,
    TreeView.Items[0], FCompiler);
  FCompilerNode.ImageIndex := 21;
  FCompilerNode.SelectedIndex := 21;
  FCompilerNode.StateIndex := 21;
  //  AddHeaderNode;
  with AddCocoNode('Options', FCompilerNode, nil) do
  begin
    ImageIndex := 23;
    SelectedIndex := 23;
    StateIndex := 23;
  end;
  //  AddOriginalATGNode;
  AddFramesNode;
  //  AddChangeHistoryNode;
  AddCommentsNode;
end;

procedure TCOCOTreeBuilder.AddDelphiNode;
begin
  FDelphiNode := AddCocoNode('Delphi', FCompilerNode, FCompiler);
  FDelphiNode.ImageIndex := 7;
  FDelphiNode.SelectedIndex := 7;
  FDelphiNode.StateIndex := 7;


  fUsesNode := AddCocoNode('Uses', FDelphiNode, FCompiler);
  fUsesNode.ImageIndex := 14;
  fUsesNode.SelectedIndex := 14;
  fUsesNode.StateIndex := 14;
  fGlobalNode := AddCocoNode('Global', FDelphiNode, FCompiler);
  fGlobalNode.ImageIndex := 14;
  fGlobalNode.SelectedIndex := 14;
  fGlobalNode.StateIndex := 14;
  fPrivateNode := AddCocoNode('Private', FDelphiNode, FCompiler);
  fPrivateNode.ImageIndex := 14;
  fPrivateNode.SelectedIndex := 14;
  fPrivateNode.StateIndex := 14;

  //AddDelphiMembers(fPrivateNode, FCompiler.DelphiSection.PrivateDecs);
  fProtectedNode := AddCocoNode('Protected', FDelphiNode, FCompiler);
  fProtectedNode.ImageIndex := 14;
  fProtectedNode.SelectedIndex := 14;
  fProtectedNode.StateIndex := 14;

  //AddDelphiMembers(fProtectedNode, FCompiler.DelphiSection.ProtectedDecs);
  fPublicNode := AddCocoNode('Public', FDelphiNode, FCompiler);
  fPublicNode.ImageIndex := 14;
  fPublicNode.SelectedIndex := 14;
  fPublicNode.StateIndex := 14;

  //AddDelphiMembers(fPublicNode, FCompiler.DelphiSection.PublicDecs);
  fPublishedNode := AddCocoNode('Published', FDelphiNode, FCompiler);
  fPublishedNode.ImageIndex := 14;
  fPublishedNode.SelectedIndex := 14;
  fPublishedNode.StateIndex := 14;
  //AddDelphiMembers(fPublishedNode, FCompiler.DelphiSection.PublishedDecs);

  fCreateNode := AddCocoNode('Create', FDelphiNode, FCompiler);
  fCreateNode.ImageIndex := 14;
  fCreateNode.SelectedIndex := 14;
  fCreateNode.StateIndex := 14;

  fDestroyNode := AddCocoNode('Destroy', FDelphiNode, FCompiler);
  fDestroyNode.ImageIndex := 14;
  fDestroyNode.SelectedIndex := 14;
  fDestroyNode.StateIndex := 14;
  AddErrorsNode;

  fArbitraryCodeNode := AddCocoNode('Arbitrary Code', FDelphiNode, FCompiler);
  fArbitraryCodeNode.ImageIndex := 14;
  fArbitraryCodeNode.SelectedIndex := 14;
  fArbitraryCodeNode.StateIndex := 14;
end;

procedure TCOCOTreeBuilder.AddIgnoreNode;
begin
  fIgnoreNode := AddCocoNode('Ignore', FCompilerNode, FCompiler);
  fIgnoreNode.ImageIndex := 16;
  fIgnoreNode.SelectedIndex := 16;
  fIgnoreNode.StateIndex := 16;
end;

procedure TCOCOTreeBuilder.AddPragmasNode;
  //var
  //  idx: integer;
  //  S: string;
  //  ct: TCocoToken;
begin
  fPragmasNode := AddCocoNode('Pragmas', FCompilerNode, FCompiler);
  fPragmasNode.ImageIndex := 15;
  fPragmasNode.SelectedIndex := 15;
  fPragmasNode.StateIndex := 15;
  //  with FCompiler.PragmasSection do
  //    for idx := 0 to PragmaCount - 1 do
  //    begin
  //      ct := GetPragmaByIndex(idx);
  //      S := ct.Symbol;
  //      AddCocoNode(S, fPragmasNode, nil);
  //    end;
end;

procedure TCOCOTreeBuilder.AddPrimaryNode;
var 
  TN: TTreeNode;
begin
  if (TreeView.Items.Count = 0) then
  begin
    TN := AddCocoNode('Parser Library', nil, nil);
    TN.ImageIndex := 30;
    TN.SelectedIndex := 30;
    TN.StateIndex := 30;
  end;
end;

procedure TCOCOTreeBuilder.AddProductionsNode;
var
  //  idx, jdx: integer;
  idx: integer;
  //  fCurrentProdNode: TTreeNode;
  fCurrProd: TCocoProduction;
begin
  fProductionsNode := AddCocoNode('Productions', FCompilerNode, FCompiler);
  fProductionsNode.ImageIndex := 18;
  fProductionsNode.SelectedIndex := 18;
  fProductionsNode.StateIndex := 18;
  // 20-03-02 - Discontinued: Showing each Path Node for a production
  // Add the child nodes - each production
  for idx := 0 to fCompiler.Productions.Count - 1 do
  begin
    fCurrProd := fCompiler.Productions.GetProduction(idx);
    //fCurrentProdNode :=
    AddCocoNode(fCurrProd.ProductionName, fProductionsNode, nil);
    //    for jdx := 0 to fCurrProd.GetPathCount - 1 do
    //    begin
    //      AddCocoNode('Path ' + IntToStr(jdx), fCurrentProdNode, nil);
    //    end;
  end;
end;

procedure TCOCOTreeBuilder.AddTokensNode;
  //var
  //  idx: integer;
  //  S: string;
  //  ct: TCocoToken;
begin
  fTokensNode := AddCocoNode('Tokens', FCompilerNode, FCompiler);
  fTokensNode.ImageIndex := 17;
  fTokensNode.SelectedIndex := 17;
  fTokensNode.StateIndex := 17;
  //  with FCompiler.TokenSection do
  //    for idx := 0 to TokenCount - 1 do
  //    begin
  //      ct := GetTokenByIndex(idx);
  //      S := ct.Symbol;
  //      AddCocoNode(S, fTokensNode, nil).ImageIndex := 25;
  //    end;
end;



procedure TCOCOTreeBuilder.SetCompiler(const Value: TCompiler);
begin
  FCompiler := Value;
end;

procedure TCOCOTreeBuilder.SetTreeView(const Value: TTreeView);
begin
  FTreeView := Value;
end;


//procedure TCOCOTreeBuilder.AddOriginalATGNode;
//begin
//  with AddCocoNode('Untouched ATG', FCompilerNode, nil) do
//  begin
//    ImageIndex := 27;
//    SelectedIndex := 27;
//    StateIndex := 27;
//  end;
//end;

procedure TCOCOTreeBuilder.AddFramesNode;
var
  TN: TTreeNode;
begin
  TN := AddCocoNode('Frames', FCompilerNode, nil);
  with TN do
  begin
    ImageIndex := 24;
    SelectedIndex := 24;
    StateIndex := 24;
  end;
end;

procedure TCOCOTreeBuilder.KillCompilerNode;
var
  idx: integer;
  cmplr: TCompiler;
  TN: TTreeNode;
begin
  for idx := 0 to TreeView.Items[0].Count - 1 do
  begin
    TN := TreeView.Items[0].Item[idx];
    Assert(TN.Data <> nil, 'Treenode should hold a compiler reference.');
    cmplr := TN.Data;
    if (cmplr.FileName = FCompiler.Filename) then
    begin
      TN.DeleteChildren;
      TreeView.Items.Delete(TN);
      Exit;
    end;
  end;
end;

//procedure TCOCOTreeBuilder.AddChangeHistoryNode;
//var
//  TN: TTreeNode;
//begin
//  TN := AddCocoNode('Change History', FCompilerNode, nil);
//  with TN do
//  begin
//    ImageIndex := 20;
//    SelectedIndex := 20;
//    StateIndex := 20;
//  end;
//end;

procedure TCOCOTreeBuilder.AddCommentsNode;
begin
  with AddCocoNode('Comments', FCompilerNode, nil) do
  begin
    ImageIndex := 5;
    SelectedIndex := 5;
    StateIndex := 5;
  end;
end;

//procedure TCOCOTreeBuilder.AddHeaderNode;
//begin
//  with AddCocoNode('Header', FCompilerNode, nil) do
//  begin
//    ImageIndex := 22;
//    SelectedIndex := 22;
//    StateIndex := 22;
//  end;
//end;

procedure TCOCOTreeBuilder.SetArbitraryCodeNode(const Value: TTreeNode);
begin
  FArbitraryCodeNode := Value;
end;

//procedure TCOCOTreeBuilder.AddDelphiMembers(Node: TTreeNode; MemberText: string);
//var
//  idx, cnt: integer;
//  S: string;
//begin
//  cnt := StrColumns(MemberText, #13#10);
//  for idx := 1 to cnt do
//  begin
//    S := Trim(StrExtract(MemberText, idx, #13#10));
//    if (S <> '') then
//    begin
//      S := CaseInsensitiveReplaceAll(S, 'procedure ', '');
//      S := CaseInsensitiveReplaceAll(S, 'function ', '');
//      S := CaseInsensitiveReplaceAll(S, 'property ', '');
//      S := CaseInsensitiveReplaceAll(S, 'constructor ', '');
//      S := CaseInsensitiveReplaceAll(S, 'destructor ', '');
//      S := StrExtract(s, 1, '(');
//      S := StrExtract(s, 1, ';');
//      S := Trim(StrExtract(s, 1, ':'));
//      AddCocoNode(S, Node, nil);
//    end;
//  end;
//end;


procedure TCOCOTreeBuilder.AddErrorsNode;
begin
  fErrorsNode := AddCocoNode('Errors', FDelphiNode, FCompiler);
  fErrorsNode.ImageIndex := 29;
  fErrorsNode.SelectedIndex := 29;
  fErrorsNode.StateIndex := 29;
end;

procedure TCOCOTreeBuilder.AddNamesNode;
begin
  fNamesNode := AddCocoNode('Names', FCompilerNode, FCompiler);
  fNamesNode.ImageIndex := 32;
  fNamesNode.SelectedIndex := 32;
  fNamesNode.StateIndex := 32;
end;

function TCOCOTreeBuilder.GetProdCount: Integer;
begin
  Result := fProductionsNode.Count;
end;

function TCOCOTreeBuilder.GetProdNodeNamesList: string;
var 
  idx: integer;
begin
  for idx := 0 to ProdCount - 1 do
  begin
    Result := Result + fProductionsNode.Item[idx].Text;
  end;
end;

end.
