{-----------------------------------------------------------------------------
 Unit Name: CocoProductionPath
 Author:    Administrator
Purpose: Container for GUI-model of ATG File - holds a path and is owned by a production
-----------------------------------------------------------------------------}


unit CocoProductionPath;

interface

uses
  Windows, Messages, SysUtils, Classes, CocoFactor,
  pbStrLib, jclSysUtils;

type
  TCocoProductionPath = class(TComponent)
  private
    FPathBody: string;
    FPathName: string;
    fFactorList: TList;
    //    FIsWeak: boolean;
    FInsideBrace: boolean;
    FIsNoneOrMany: boolean;
    FIsNoneOrOne: boolean;
    fsaMinIndent: Integer;
    procedure SetPathBody(const Value: string);
    procedure SetPathName(const Value: string);
    //    procedure SetIsWeak(const Value: boolean);
    //    procedure SetInsideBrace(const Value: boolean);
    //    procedure SetIsNoneOrMany(const Value: boolean);
    //    procedure SetIsNoneOrOne(const Value: boolean);
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(Aowner: TComponent); override;
    destructor Destroy; override;
    function AddFactor: TCocoFactor;
    function GetFactor(const idx: Integer): TCocoFactor;
    procedure DeleteFactor(const idx: Integer);
    function GetATG: string;
  published
    { Published declarations }
    property PathName: string read FPathName write SetPathName;
    property PathBody: string read FPathBody write SetPathBody;
    // New - Indent properties for semantic actions
    property saMinIndent: Integer read fsaMinIndent write fsaMinIndent;

    //    property IsWeak: boolean read FIsWeak write SetIsWeak;
    //    property IsNoneOrMany: boolean read FIsNoneOrMany write SetIsNoneOrMany;
    //    property IsNoneOrOne: boolean read FIsNoneOrOne write SetIsNoneOrOne;
    //    property InsideBrace: boolean read FInsideBrace write SetInsideBrace;
  end;

implementation

{ TCocoProductionPath }

function TCocoProductionPath.AddFactor: TCocoFactor;
begin
  Result := TCocoFactor.Create(Self);
  Assert(Result <> nil);
  Assert(ffactorList <> nil);
  fFactorList.Add(Result);
  Result.saMinIndent := fsaMinIndent;
end;

constructor TCocoProductionPath.Create(Aowner: TComponent);
begin
  inherited;
  fFactorList := TList.Create;
end;

procedure TCocoProductionPath.DeleteFactor(const idx: Integer);
var
  cf: TCocoFactor;
begin
  cf := fFactorList[idx];
  fFactorList.Delete(idx);
  cf.Free;
end;

destructor TCocoProductionPath.Destroy;
begin
  FreeObjectList(fFactorList);
  inherited;
end;

function TCocoProductionPath.GetATG: string;
var
  S, S1: string;
  idx, CurLen: integer;
  cf: TCocoFactor;
begin
  // iterate through the factors, outputting the atg for each one
  CurLen := 0;
  for idx := 0 to fFactorList.Count - 1 do
  begin
    cf := fFactorList[idx];
    cf.saMinIndent := fsaMinIndent;
    if (idx = 0) then
    begin
      if (FIsNoneOrMany) then
      begin
        cf.IsNoneOrMany := True;
      end
      else if (FInsideBrace) then
      begin
        cf.InsideBrace := True;
      end
      else if (FIsNoneOrOne) then
      begin
        cf.IsNoneOrOne := True;
      end;
      //S := S  + '        ' ;
    end;

    if (idx = fFactorList.Count - 1) then
      if (FIsNoneOrMany) then
      begin
        cf.DoesCloseNoneOrMany := True;
      end
      else if (FInsideBrace) then
      begin
        cf.DoesCloseInsideBrace := True;
      end
      else if (FIsNoneOrOne) then
      begin
        cf.DoesCloseNoneOrOne := True;
      end;
    if (idx < fFactorList.Count - 1) then
      S1 := cf.GetATG
    else
      S1 := KillFInalDelim(cf.GetATG, #13#10);

    if CurLen>50 then
    Begin
      S := S + CRLF+S1;
      CurLen := 0;
    End
    Else
    Begin
      CurLen := CurLen + Length(S1);
      S := S + S1;
    End;
  end;
//  Result := '        '+ReplaceAll(S,#13#10,#13#10'        ');
  Result := Trim(S);
end;

function TCocoProductionPath.GetFactor(const idx: Integer): TCocoFactor;
begin
  Result := fFactorList[idx];
end;

procedure TCocoProductionPath.SetPathBody(const Value: string);
begin
  FPathBody := Value;
end;

procedure TCocoProductionPath.SetPathName(const Value: string);
begin
  FPathName := Value;
end;

end.
