{-----------------------------------------------------------------------------
 Unit Name: DelphiDeclarations
 Author:     pat connors
Purpose: Container for GUI-model of ATG File - This holds all the "Delphi" sections
-----------------------------------------------------------------------------}


unit DelphiDeclarations;

interface

uses
  Windows, Messages, SysUtils, Classes, pbStrLib, Forms;

type
  TDelphiDeclarations = class(TComponent)
  private
    FPublishedDecs: string;
    FDestroySrc: string;
    FPublicDecs: string;
    FUsesClause: string;
    FPrivateDecs: string;
    FProtectedDecs: string;
    FCreateSrc: string;
    FArbitraryCodeSection: string;
    FTypeDeclarations: string;
    FConstantDecs: string;
    FCOMPILERFRAME: string;
    FPARSERFRAME: string;
    FSCANNERFRAME: string;
    FIsEditing: boolean;
    fIsEditingPublishedDecs: boolean;
    fIsEditingDestroySrc: boolean;
    fIsEditingUsesClause: boolean;
    fIsEditingProtectedDecs: boolean;
    fIsEditingCreateSrc: boolean;
    fIsEditingPrivateDecs: boolean;
    fIsEditingTypeDeclarations: boolean;
    fIsEditingArbitraryCodeSection: boolean;
    fIsEditingPublicDecs: boolean;
    FFileName: string;
    FTargetDirectory: string;
    FIsEditingTargetDir: boolean;
    FErrorsSource: string;
    FIsEditingErrorsSource: boolean;
    procedure SetCreateSrc(const Value: string);
    procedure SetDestroySrc(const Value: string);
    procedure SetPrivateDecs(const Value: string);
    procedure SetProtectedDecs(const Value: string);
    procedure SetPublicDecs(const Value: string);
    procedure SetPublishedDecs(const Value: string);
    procedure SetUsesClause(const Value: string);
    procedure SetArbitraryCodeSection(const Value: string);
    procedure SetTypeDeclarations(const Value: string);
    procedure SetConstantDecs(const Value: string);
    procedure SetCOMPILERFRAME(const Value: string);
    procedure SetPARSERFRAME(const Value: string);
    procedure SetSCANNERFRAME(const Value: string);
    function GetCOMPILERFRAME: string;
    function GetPARSERFRAME: string;
    function GetSCANNERFRAME: string;
    procedure SetIsEditing(const Value: boolean);
    procedure SetFileName(const Value: string);
    procedure SetTargetDirectory(const Value: string);
    procedure SetIsEditingTargetDir(const Value: boolean);
    procedure SetErrorsSource(const Value: string);
    procedure SetIsEditingErrorsSource(const Value: boolean);
    procedure SetIsEditingTypeDeclarations(const Value: boolean);
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetATG: string;
  published
    { Published declarations }
    property UsesClause: string read FUsesClause write SetUsesClause;
    property PrivateDecs: string read FPrivateDecs write SetPrivateDecs;
    property ProtectedDecs: string read FProtectedDecs write SetProtectedDecs;
    property PublicDecs: string read FPublicDecs write SetPublicDecs;
    property PublishedDecs: string read FPublishedDecs write SetPublishedDecs;
    property CreateSrc: string read FCreateSrc write SetCreateSrc;
    property DestroySrc: string read FDestroySrc write SetDestroySrc;
    property ErrorsSource: string read FErrorsSource write SetErrorsSource;
    property TypeDeclarations: string read FTypeDeclarations write SetTypeDeclarations;
    property ArbitraryCodeSection: string read FArbitraryCodeSection          
      write SetArbitraryCodeSection;
    property ConstantDecs: string read FConstantDecs write SetConstantDecs;
    property COMPILERFRAME: string read GetCOMPILERFRAME write SetCOMPILERFRAME;
    property PARSERFRAME: string read GetPARSERFRAME write SetPARSERFRAME;
    property SCANNERFRAME: string read GetSCANNERFRAME write SetSCANNERFRAME;

    property IsEditing: boolean read FIsEditing write SetIsEditing;
    property IsEditingPublishedDecs: boolean          
      read fIsEditingPublishedDecs write fIsEditingPublishedDecs;
    property IsEditingDestroySrc: boolean read fIsEditingDestroySrc          
      write fIsEditingDestroySrc;
    property IsEditingPublicDecs: boolean read fIsEditingPublicDecs       
      write fIsEditingPublicDecs;
    property IsEditingTargetDir: boolean read FIsEditingTargetDir       
      write SetIsEditingTargetDir;
    property IsEditingUsesClause: boolean read fIsEditingUsesClause          
      write fIsEditingUsesClause;
    property IsEditingPrivateDecs: boolean read fIsEditingPrivateDecs          
      write fIsEditingPrivateDecs;
    property IsEditingProtectedDecs: boolean          
      read fIsEditingProtectedDecs write fIsEditingProtectedDecs;
    property IsEditingCreateSrc: boolean read fIsEditingCreateSrc write fIsEditingCreateSrc;
    property IsEditingArbitraryCodeSection: boolean    
      read fIsEditingArbitraryCodeSection write fIsEditingArbitraryCodeSection;
    property IsEditingTypeDeclarations: boolean   
      read fIsEditingTypeDeclarations write SetIsEditingTypeDeclarations;

    property IsEditingErrorsSource: boolean    
      read FIsEditingErrorsSource write SetIsEditingErrorsSource;
    property FileName: string read FFileName write SetFileName;
    property TargetDirectory: string read FTargetDirectory write SetTargetDirectory;
  end;

implementation

{ TDelphiDeclarations }

constructor TDelphiDeclarations.Create(AOwner: TComponent);
begin
  inherited;
  FPublishedDecs := '';
  FDestroySrc := ''; //'    fOutputStream.Free;'+#13+#10 ;
  FPublicDecs := '';
  FUsesClause := '';
  FPrivateDecs := '';
  FProtectedDecs := '';
  FCreateSrc := ''; //'    fOutputStream := TMemoryStream.Create;'+#13+#10 ;
  FErrorsSource := '';
  FArbitraryCodeSection := '';
end;

destructor TDelphiDeclarations.Destroy;
begin
  inherited;
end;

function TDelphiDeclarations.GetATG: string;
var
  S, S1: string;
begin
  if not fIsEditing then
  begin
    if not fIsEditingUsesClause then
    begin
      if Trim(FUsesClause) <> ';' then
        Result := 'USES ' + Trim(FUsesClause) + CRLF
      else
        Result := CRLF;
    end
    else
      Result := 'USES ' + ModTextTag + CRLF;

    Result := Result + 'DELPHI' + CRLF;
    if not fIsEditingTypeDeclarations then
      Result := Result + 'GLOBAL' + CRLF + '  ' + Trim(FTypeDeclarations) + CRLF
    else
      Result := Result + 'GLOBAL' + CRLF + ModTextTag + CRLF;

    Result := Result + 'TARGETDIRECTORY' + CRLF + '  ' + Trim(FTargetDirectory) + CRLF;



    S := ExtractFilePath(Application.ExeName) + 'Frames\';
    S1 := KillFinalDelim(ExtractFilePath(ffilename), '\');
    if S1 = '' then
      S1 := ExtractFilePath(Application.ExeName) + 'Output';

    if (FCOMPILERFRAME = '') then
      FCOMPILERFRAME := S + 'pbCOMPILER.FRM';

    Result := Result + 'COMPILERFRAME' + CRLF + '  ' + Trim(FCOMPILERFRAME) + CRLF;

    if (FPARSERFRAME = '') then
      FPARSERFRAME := S + 'pbPARSER.FRM';
    Result := Result + 'PARSERFRAME' + CRLF + '  ' + Trim(FPARSERFRAME) + CRLF;
    if (FSCANNERFRAME = '') then
      FSCANNERFRAME := S + 'pbSCANNER.FRM';
    Result := Result + 'SCANNERFRAME' + CRLF + '  ' + Trim(FSCANNERFRAME) + CRLF;
    if not FIsEditingPrivateDecs then
      Result := Result + 'PRIVATE' + CRLF + Trim(FPrivateDecs) + CRLF
    else
      Result := Result + 'PRIVATE' + CRLF + ' ' + ModTextTag + CRLF;
    if not FIsEditingProtectedDecs then
      Result := Result + 'PROTECTED' + CRLF + Trim(FProtectedDecs) + CRLF
    else
      Result := Result + 'PROTECTED' + CRLF + ' ' + ModTextTag + CRLF;
    if not FIsEditingPublicDecs then
      Result := Result + 'PUBLIC' + CRLF + Trim(FPublicDecs) + CRLF
    else
      Result := Result + 'PUBLIC' + CRLF + ' ' + ModTextTag + CRLF;
    if not FIsEditingCreateSrc then
      Result := Result + 'CREATE' + CRLF + Trim(FCreateSrc) + CRLF
    else
      Result := Result + 'CREATE' + CRLF + '  ' + ModTextTag + CRLF;
    if not FIsEditingDestroySrc then
      Result := Result + 'DESTROY' + CRLF + Trim(FDestroySrc) + CRLF
    else
      Result := Result + 'DESTROY' + CRLF + ' ' + ModTextTag + CRLF;

    if not FIsEditingErrorsSource then
      Result := Result + 'ERRORS' + CRLF + Trim(FErrorsSource) + CRLF
    else
      Result := Result + 'ERRORS' + CRLF + ' ' + ModTextTag + CRLF;

    if not FIsEditingPublishedDecs then
      Result := Result + 'PUBLISHED' + CRLF + Trim(FPublishedDecs) + CRLF
    else
      Result := Result + 'PUBLISHED' + CRLF + ' ' + ModTextTag + CRLF;
    Result := Result + 'END_DELPHI' + CRLF + CRLF;
    Result := Result;
    if not fIsEditingArbitraryCodeSection then
      Result := Result + Trim(FArbitraryCodeSection) + CRLF +
        CRLF
    else
      Result := Result + ModTextTag + CRLF;
  end
  else
    Result := ModTextTag + CRLF;
end;

function TDelphiDeclarations.GetCOMPILERFRAME: string;
begin
  Result := fCompilerFrame;
end;


function TDelphiDeclarations.GetPARSERFRAME: string;
begin
  Result := FPARSERFRAME;
end;


function TDelphiDeclarations.GetSCANNERFRAME: string;
begin
  Result := FSCANNERFRAME;
end;


procedure TDelphiDeclarations.SetArbitraryCodeSection(const Value: string);
begin
  FArbitraryCodeSection := Value;
end;

procedure TDelphiDeclarations.SetCOMPILERFRAME(const Value: string);
begin
  FCOMPILERFRAME := Value;
end;


procedure TDelphiDeclarations.SetConstantDecs(const Value: string);
begin
  FConstantDecs := Value;
end;

procedure TDelphiDeclarations.SetCreateSrc(const Value: string);
begin
  FCreateSrc := Value;
end;

procedure TDelphiDeclarations.SetDestroySrc(const Value: string);
begin
  FDestroySrc := Value;
end;

procedure TDelphiDeclarations.SetErrorsSource(const Value: string);
begin
  FErrorsSource := Value;
end;

procedure TDelphiDeclarations.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure TDelphiDeclarations.SetIsEditing(const Value: boolean);
begin
  FIsEditing := Value;
end;

procedure TDelphiDeclarations.SetIsEditingErrorsSource(const Value: boolean);
begin
  FIsEditingErrorsSource := Value;
end;

procedure TDelphiDeclarations.SetIsEditingTargetDir(const Value: boolean);
begin
  FIsEditingTargetDir := Value;
end;

procedure TDelphiDeclarations.SetIsEditingTypeDeclarations(const Value: boolean);
begin
  fIsEditingTypeDeclarations := Value;
end;

procedure TDelphiDeclarations.SetPARSERFRAME(const Value: string);
begin
  FPARSERFRAME := Value;
end;


procedure TDelphiDeclarations.SetPrivateDecs(const Value: string);
begin
  FPrivateDecs := Value;
end;

procedure TDelphiDeclarations.SetProtectedDecs(const Value: string);
begin
  FProtectedDecs := Value;
end;

procedure TDelphiDeclarations.SetPublicDecs(const Value: string);
begin
  FPublicDecs := Value;
end;

procedure TDelphiDeclarations.SetPublishedDecs(const Value: string);
begin
  FPublishedDecs := Value;
end;

procedure TDelphiDeclarations.SetSCANNERFRAME(const Value: string);
begin
  FSCANNERFRAME := Value;
end;

procedure TDelphiDeclarations.SetTargetDirectory(const Value: string);
begin
  FTargetDirectory := Value;
end;

procedure TDelphiDeclarations.SetTypeDeclarations(const Value: string);
begin
  FTypeDeclarations := Value;
end;

procedure TDelphiDeclarations.SetUsesClause(const Value: string);
begin
  FUsesClause := Trim(Value);
end;

end.
